/***************************************************************
  TORNADO-6x utilities (C6x DSP Environment).
  (C) MicroLAB Systems Ltd, 1998-2000
****************************************************************/

#define	ON			1
#define	OFF			0

//================ T6x Memory/IO Areas ===============

	//----------- SBSRAM area @CE-0
#define	T6X_SBSRAM_SADDR	0x00000000
#define	T6X_SBSRAM_EADDR	0x000fffff
#define	T6X_SBSRAM_LEN		0x00100000

	//----------- FLASH/EPROM area @CE-1
#define	T6X_FLASH_SADDR		0x01000000	
#define	T6X_FLASH_EADDR		0x013fffff
#define	T6X_FLASH_LEN		0x00400000

	//----------- AUX I/O @CE-3
#define T6X_MLOCK_ADDR		0x03000004	// r/w via D0
#define T6X_MH_RQ_ADDR		0x03000000	// w-only, D0 is ignored
#define	T6X_WDT_RESET_ADDR	0x03100000	// w-only, D0 is ignored
#define T6X_WDT_ENABLE_ADDR	0x03100004	// r/w via D0
#define T6X_XRESET_ADDR		0x03000008	// r/w via D0
#define T6X_PXSX_RUN_ADDR	0x03000008	// r/w via D0/D1/D2
#define T6X_PIOX_SADDR		0x03200000	// r/w, D0..D31 (8/16/32-bit cycles)
#define T6X_PIOX_EADDR		0x033fffff
#define T6X_PIOX_LEN		0x00200000

	//------ recommended EMIF configurations ----------
#define T6X_EMIF_GCR_DATA	0x3078          // 1/2 CLK SBSRAM
#define T6X_EMIF_CE0_DATA	0x40		// SBSRAM
#define T6X_EMIF_CE1_DATA	0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define T6X_EMIF_CE2_DATA	0xffffff23	// reserved   (all=max_time_len)
#define T6X_EMIF_CE3_DATA	0x31b3c623	// PIOX/IOX area (stb=6, setup=3,hold=3)
//================================================================


//********** Macros for Aux I/O MLOCK/WDI/MH_RQ Control *************

#define T6X_set_mh_rq()		{*(unsigned *) T6X_MH_RQ_ADDR=0;}
#define T6X_wdt_reset()		{*(unsigned *) T6X_WDT_RESET_ADDR=0;}
#define T6X_set_mlock(v)	{*(unsigned *) T6X_MLOCK_ADDR=((unsigned)v &1);}
#define T6X_mlock()		((*(unsigned *) T6X_MLOCK_ADDR) &1)
#define T6X_set_wdt_enable(v)	{*(unsigned *) T6X_WDT_ENABLE_ADDR=((unsigned)v &1);}
#define T6X_wdt_enable()	((*(unsigned *) T6X_WDT_ENABLE_ADDR) &1)

/* XRESET control: T6x rev.1.x only
   (common PIOX/SIOX RESET signal)
   note: XRESET is generated as OR between DSP_RESET signal and _XRESET port
*/
#define T6X_set_pxsx_run(v)	{*(unsigned *) T6X_PXSX_RUN_ADDR=((unsigned)v &7);}
#define T6X_pxsx_run()		((*(unsigned *) T6X_PXSX_RUN_ADDR) &7)
#define T6X_set_px_run(v)	{*(unsigned *) T6X_PXSX_RUN_ADDR=((unsigned)v &1);}
#define T6X_px_run()		((*(unsigned *) T6X_PXSX_RUN_ADDR) &1)
#define T6X_set_sxa_run(v)	{*(unsigned *) T6X_PXSX_RUN_ADDR=((((unsigned)v)<<1) &2);}
#define T6X_sxa_run()		(((*(unsigned *) T6X_PXSX_RUN_ADDR)>>1) &1)
#define T6X_set_sxb_run(v)	{*(unsigned *) T6X_PXSX_RUN_ADDR=((((unsigned)v)<<2) &4);}
#define T6X_sxb_run()		(((*(unsigned *) T6X_PXSX_RUN_ADDR)>>2) &1)


//====================== HPI port specific macros ======================
#define	_C6x_HPIC_ADDR					0x01880000

#define	_C6X_HPIC_DSPINT_BITMASK		0x2
#define	_C6X_HPI_DSPINT_BITMASK			0x2
#define	_C6X_HPIC_HINT_BITMASK			0x4	
#define	_C6X_HPI_HINT_BITMASK			0x4	

#define T6X_hpic_dspint()		(((*(unsigned *) _C6x_HPIC_ADDR)>>1) &0x1)
#define T6X_hpic_hint()			(((*(unsigned *) _C6x_HPIC_ADDR)>>2) &0x1)
#define T6X_clr_hpic_dspint()	(*(unsigned *) _C6x_HPIC_ADDR)= _C6X_HPI_DSPINT_BITMASK
#define T6X_set_hpic_hint()		(*(unsigned *) _C6x_HPIC_ADDR)= _C6X_HPIC_HINT_BITMASK
	
	// old function names aliases
#define T6X_dspint()			T6X_hpic_dspint()
#define T6X_hint()				T6X_hpic_hint()
#define T6X_clr_dspint()		T6X_clr_hpic_dspint()
#define T6X_set_hint()			T6X_set_hpic_hint()

//======================================================================


//--------------- DSP/board type ID macros -----------------------------
#define T62_BOARD				((CSR&0x02000000)==0)
#define T67_BOARD				((CSR&0x02000000)!=0)
//----------------------------------------------------------------------
