/***************************************************************************

  TORNADO-6X Host Control Utilities. v.1.2
  (C) MicroLAB Systems Ltd, 1998

  note: this file include definitions and function declarations only
	and should be used together with T6X_H.C file.

****************************************************************************/

#define	T6X_ON				1
#define	T6X_OFF				0

typedef unsigned char BYTE ; 

#define	T6X_OK				0		// error codes
#define	T6X_PARAM_ERR			-1
#define	T6X_SMP_OFF_ERR			-3
#define	T6X_HW_ERR			-5
#define	T6X_NOT_OPEN_ERR		-10
#define	T6X_NOT_FOUND_ERR		-11
#define T6X_INV_CMND_ERR		-12


//================= T6x specific definitions ==============================

#define T6X_SRAM_SADDR			0L
#define T62_SRAM_EADDR			0x1fffffL  // T62/T67 2 banks 256Kx32 each
#define T67_SRAM_EADDR			0x1fffffL
#define T62MX_SRAM_EADDR		0xfffffL  // T62MX/T67MX 1 bank 256Kx32 each
#define T67MX_SRAM_EADDR		0xfffffL
#define T6X_SRAM_EADDR_MAX		0x1fffffL  // max T6X SRAM-eaddr

#define T6X_FLASH_SADDR			0L
#define T6X_FLASH_EADDR			0x3fffffL  // max 1Mx8 FLASH allocated as LSB of on 32-bit words
#define T6X_FLASH_EADDR_MAX		0x3fffffL 

#define T6X_IO_SADDR			0L
#define T62_IO_EADDR			0x1fffffL  // T62/T67 32-bit PIOX
#define T67_IO_EADDR			0x1fffffL
#define T62MX_IO_EADDR			0x1ffffL   // T62MX/T67MX 16-bit PIOX-16
#define T67MX_IO_EADDR			0x1ffffL
#define T6X_IO_EADDR_MAX		0x1fffffL  // max T6X IO-eaddr

	//----- define C6x DSP environment params
#define T6X_DSP_ONCHIP_PRAM_SADDR	0x01400000L  // C6x on-chip P-RAM
#define T6X_DSP_ONCHIP_PRAM_EADDR	0x0140ffffL
#define T6X_DSP_ONCHIP_DRAM_SADDR	0x80000000L  // C6x on-chip D-RAM
#define T6X_DSP_ONCHIP_DRAM_EADDR	0x8000ffffL
#define T6X_DSP_SRAM_SADDR		0x00000000L  // C6x SBSRAM
#define T6X_DSP_SRAM_EADDR_MAX		0x001fffffL
#define T62_DSP_SRAM_EADDR		0x001fffffL
#define T67_DSP_SRAM_EADDR		0x001fffffL
#define T62MX_DSP_SRAM_EADDR		0x000fffffL
#define T67MX_DSP_SRAM_EADDR		0x000fffffL
#define T6X_DSP_FLASH_SADDR		0x01000000L  // C6x SBSRAM
#define T6X_DSP_FLASH_EADDR		0x013fffffL
#define T6X_DSP_ONCHIP_RG_SADDR		0x01800000L  // C6x DSP on-chip registers
#define T6X_DSP_ONCHIP_RG_EADDR		0x01FfffffL
#define T6X_DSP_AUXIO_SADDR		0x03000000L  // C6x AUX-IO area
#define T6X_DSP_AUXIO_EADDR		0x031fffffL
#define T6X_DSP_PIOX_SADDR		0x03200000L  // C6x PIOX
#define T6X_DSP_PIOX_EADDR_MAX		0x033fffffL
#define T62_DSP_PIOX_EADDR		0x033fffffL
#define T67_DSP_PIOX_EADDR		0x033fffffL
#define T62MX_DSP_PIOX16_EADDR		0x0301ffffL
#define T67MX_DSP_PIOX16_EADDR		0x0301ffffL


#define	T62_DEV_ID			0x6200
#define	T67_DEV_ID			0x6300
#define	T62MX_DEV_ID			0x6400
#define	T67MX_DEV_ID			0x6500

#define	T6X_IO_BADDR_CNF_QTY		8
#define	T6X_SMP_SEGM_CNF_QTY		8
#define	T6X_IO_BADDR_DEFAULT		0x350
#define	T6X_SMP_SEGM_DEFAULT		0xd800
#define T6X_SMP_LEN			0x8000
#define	T6X_SMP_OFF			0


unsigned T6X_io_baddr();
unsigned T6X_smp_segm();
unsigned T6X_id();
unsigned T6X_emu_io_baddr();
unsigned T6X_io_baddr_table(int);
unsigned T6X_smp_segm_table(int);
unsigned T6X_ecc_io_baddr_table(int);
unsigned T6X_uecm_io_baddr_table(int);


//------------------- T6X I/O registers ---------------------- 
#define	 T6X_SMP_RG			(T6X_io_baddr())  // T6X I/O registers
#define  T6X_SMP_RG_LSB			(T6X_io_baddr())
#define  T6X_SMP_RG_MSB			(T6X_io_baddr()+1)
#define  T6X_CNTR_RG			(T6X_io_baddr()+2)
#define  T6X_FDATA_RG			(T6X_io_baddr()+3)
#define  T6X_FSEL_RG			(T6X_io_baddr()+0x403)
#define  T6X_HPIC_RG_LSW		(T6X_io_baddr()+8)
#define  T6X_HPIC_RG_MSW		(T6X_io_baddr()+0xa)
#define  T6X_HPIA_RG_LSW		(T6X_io_baddr()+0xc)
#define  T6X_HPIA_RG_MSW		(T6X_io_baddr()+0xe)
#define  T6X_HPID_AINC_RG_LSW		(T6X_io_baddr()+0x408)
#define  T6X_HPID_AINC_RG_MSW		(T6X_io_baddr()+0x40a)
#define  T6X_HPID_RG_LSW		(T6X_io_baddr()+0x40c)
#define  T6X_HPID_RG_MSW		(T6X_io_baddr()+0x40e)

#define  T6X_SRAM_AREA_SEL		0x0000	// SMP/MSB rg area codes
#define  T6X_FLASH_AREA_SEL		0x4000
#define  T6X_IO_AREA_SEL		0xC000
#define  T6X_AREA_SEL_BITMASK		0xC000

#define T6X_MRESET_RESET 		0x0		// Control Register bits
#define T6X_MRESET_GO			0x1
#define T6X_MRESET_BITMASK		0x1

#define T6X_SB_GLOCK_BITMASK		0x4
#define T6X_SB_LOCK_BITMASK		0x8

#define T6X_SB_CCL_BYTE			0
#define T6X_SB_CCL_W16			0x10
#define T6X_SB_CCL_W32			0x20
#define T6X_SB_CCL_W32_2		0x30
#define T6X_SB_CCL_BITMASK		0x30

#define T6X_SB_ERR_IE_BITMASK		0x80
#define T6X_MH_RQ_IE_BITMASK		0x40

#define T6X_SB_ACK_FLAG_BITMASK		0x1		// SYSTEM STATUS FRG bits
#define T6X_SB_ERR_FLAG_BITMASK		0x80
#define T6X_MH_RQ_FLAG_BITMASK		0x40
#define T6X_HPI_HINT_FLAG_BITMASK	0x20
#define T6X_HPI_ERR_FLAG_BITMASK	0x2

#define T6X_SYS_STAT_FRG		0x10		
#define T6X_CLR_MH_RQ_FRG		0x20		
#define T6X_CLR_SB_ERR_FRG		0x30
#define T6X_SET_HM_RQ_FRG		0x10
#define T6X_CLR_HPI_ERR_FRG		0x40
#define T6X_HPI_IE_FRG			0x50
#define T6X_DSP_STAT_FRG		0x60
#define T6X_ISA_MI_BADDR_FRG		0xE0
#define T6X_ECC_IO_BADDR_FRG		0xE2
#define T6X_DEV_ID0_FRG			0xF0
#define T6X_DEV_ID1_FRG			0xF1

#define T6X_HPI_ERR_IE_BITMASK		0x2
#define T6X_HPI_HINT_IE_BITMASK		0x20
#define T6X_DSP_MLOCK_BITMASK		0x1
#define T6X_DSP_MRESET_BITMASK		0x2
#define T6X_DSP_PD_BITMASK		0x4


//-------------------------------------------------------------------------



//----------------------- macros for CONTROL REGISTER -----------------------------

#define T6X_CNTR_RG_DATA		inp(T6X_CNTR_RG)
#define T6X_SET_CNTR_RG(d)		outp(T6X_CNTR_RG,d)

#define T6X_MRESET_DATA			(inp(T6X_CNTR_RG) & T6X_MRESET_BITMASK)
#define T6X_SET_MRESET			outp(T6X_CNTR_RG,inp(T6X_CNTR_RG)&~T6X_MRESET_BITMASK)
#define T6X_SET_MGO			outp(T6X_CNTR_RG,inp(T6X_CNTR_RG)&~T6X_MRESET_BITMASK | T6X_MRESET_GO)

#define T6X_SB_GLOCK_DATA		(inp(T6X_CNTR_RG) &  T6X_SB_GLOCK_BITMASK)
#define T6X_SET_SB_GLOCK_ON		outp(T6X_CNTR_RG,inp(T6X_CNTR_RG) |  T6X_SB_GLOCK_BITMASK)
#define T6X_SET_SB_GLOCK_OFF		outp(T6X_CNTR_RG,inp(T6X_CNTR_RG) & ~T6X_SB_GLOCK_BITMASK)
#define T6X_SB_LOCK_DATA		(inp(T6X_CNTR_RG) &  T6X_SB_LOCK_BITMASK)
#define T6X_SET_SB_LOCK_ON		outp(T6X_CNTR_RG,inp(T6X_CNTR_RG) |  T6X_SB_LOCK_BITMASK)
#define T6X_SET_SB_LOCK_OFF		outp(T6X_CNTR_RG,inp(T6X_CNTR_RG) & ~T6X_SB_LOCK_BITMASK)

#define T6X_SB_CCL_DATA			(inp(T6X_CNTR_RG) & T6X_SB_CCL_BITMASK)
#define T6X_SET_SB_CCL_BYTE		outp(T6X_CNTR_RG,inp(T6X_CNTR_RG)&~T6X_SB_CCL_BITMASK |T6X_SB_CCL_BYTE)
#define T6X_SET_SB_CCL_W16		outp(T6X_CNTR_RG,inp(T6X_CNTR_RG)&~T6X_SB_CCL_BITMASK |T6X_SB_CCL_W16)
#define T6X_SET_SB_CCL_W32		outp(T6X_CNTR_RG,inp(T6X_CNTR_RG)&~T6X_SB_CCL_BITMASK |T6X_SB_CCL_W32)

#define T6X_SB_ERR_IE_DATA		(inp(T6X_CNTR_RG) &  T6X_SB_ERR_IE_BITMASK)
#define T6X_SET_SB_ERR_IE_ON		outp(T6X_CNTR_RG,inp(T6X_CNTR_RG) |  T6X_SB_ERR_IE_BITMASK)
#define T6X_SET_SB_ERR_IE_OFF		outp(T6X_CNTR_RG,inp(T6X_CNTR_RG) & ~T6X_SB_ERR_IE_BITMASK)

#define T6X_MH_RQ_IE_DATA		(inp(T6X_CNTR_RG) &  T6X_MH_RQ_IE_BITMASK)
#define T6X_SET_MH_RQ_IE_ON		outp(T6X_CNTR_RG,inp(T6X_CNTR_RG) |  T6X_MH_RQ_IE_BITMASK)
#define T6X_SET_MH_RQ_IE_OFF		outp(T6X_CNTR_RG,inp(T6X_CNTR_RG) & ~T6X_MH_RQ_IE_BITMASK)

#define T6X_INIT			{outpw(T6X_CNTR_RG,0);outpw(T6X_SMP_RG,0);T6X_clr_sb_err();\
					 T6X_clr_hpi_err();T6X_clr_mh_rq();T6X_set_hpi_hint_ie(T6X_OFF);T6X_set_hpi_err_ie(T6X_OFF);}


//------------------ macros for FLAGs control/status --------------------

int	T6X_set_fsel_rg_verif(unsigned frg);
int	T6X_set_frg_verif(unsigned frg,BYTE value);
BYTE	T6X_frg_data(unsigned frg);
BYTE	T6X_sys_stat_frg_data();
int	T6X_sb_err_data();
int	T6X_sb_ack_data();
int	T6X_mh_rq_data();
int	T6X_hpi_hint_data();
int	T6X_hpi_err_data();
int	T6X_set_hm_rq();
int	T6X_clr_sb_err();
int	T6X_clr_mh_rq();
int	T6X_clr_hpi_err();
int	T6X_hpi_err_ie_data();
int	T6X_hpi_hint_ie_data();
int	T6X_hpi_ie_frg_data();
int	T6X_set_hpi_hint_ie(int);
int	T6X_set_hpi_err_ie(int);
int	T6X_dsp_stat_frg_data();
int	T6X_dsp_mlock_data();
int	T6X_dsp_pd_data();
int	T6X_dsp_mreset_data();


#define T6X_FSEL_RG_DATA		inp(T6X_FSEL_RG)
#define T6X_SET_FSEL_RG(n)		outp(T6X_FSEL_RG,n)
#define	T6X_SET_FSEL_RG_VERIF(n)	T6X_set_fsel_rg_verif(n)

#define T6X_FDATA_RG_DATA		inp(T6X_FDATA_RG)
#define T6X_SET_FDATA_RG(n)		outp(T6X_FDATA_RG,n)

#define T6X_SET_FRG_VERIF(r,v)		T6X_set_frg_verif(r,v)
#define T6X_FRG_DATA(r)			T6X_frg_data(r)

#define T6X_SYS_STAT_FRG_DATA		T6X_sys_stat_frg_data()
#define T6X_SB_ERR_DATA			T6X_sb_err_data()
#define T6X_SB_ACK_DATA			T6X_sb_ack_data()
#define T6X_MH_RQ_DATA			T6X_mh_rq_data()
#define T6X_HPI_HINT_DATA		T6X_hpi_hint_data()
#define T6X_HPI_ERR_DATA		T6X_hpi_err_data()

#define T6X_SET_HM_RQ			T6X_set_hm_rq()
#define T6X_CLR_SB_ERR			T6X_clr_sb_err()
#define T6X_CLR_MH_RQ			T6X_clr_mh_rq()
#define T6X_CLR_HPI_ERR			T6X_clr_hpi_err()

#define T6X_DSP_STAT_FRG_DATA		T6X_dsp_stat_frg_data()
#define T6X_DSP_MLOCK_DATA		T6X_dsp_mlock_data()
#define T6X_DSP_MRESET_DATA		T6X_dsp_mreset_data()
#define T6X_DSP_PD_DATA			T6X_dsp_pd_data()

#define T6X_HPI_IE_FRG_DATA		T6X_hpi_ie_frg_data()
#define T6X_HPI_ERR_IE_DATA		T6X_hpi_err_ie_data()
#define T6X_HPI_HINT_IE_DATA		T6X_hpi_hint_ie_data()
#define T6X_SET_HPI_ERR_IE_ON		T6X_set_hpi_err_ie(T6X_ON)
#define T6X_SET_HPI_ERR_IE_OFF		T6X_set_hpi_err_ie(T6X_OFF)
#define T6X_SET_HPI_HINT_IE_OFF		T6X_set_hpi_hint_ie(T6X_OFF)
#define T6X_SET_HPI_HINT_IE_ON		T6X_set_hpi_hint_ie(T6X_ON)

//==================== macros for SMP ===============================

#define	T6X_SRAM_AREA			0
#define	T6X_FLASH_AREA			1
#define	T6X_IO_AREA			3

#define T6X_SET_SMP_RG(page)		outpw(T6X_SMP_RG,page)
#define T6X_SMP_RG_DATA			inpw(T6X_SMP_RG)

#define T6X_SET_SMP_SEGM(s)		T6X_set_smp_segm(s)
#define T6X_SET_SMP_OFF			T6X_set_smp_off()

#define __MK_FP__(seg,offs)		((void far *)(((unsigned long)(seg) << 16) | (unsigned)(offs)))
#define T6X_MAKE_SMP(saddr)		((unsigned)( (long)(saddr/T6X_SMP_LEN)))
#define T6X_MAKE_SMP_PTR(saddr)		(__MK_FP__(_t6x_smp_segm,(unsigned)((long)(saddr%T6X_SMP_LEN))))


int	T6X_set_smp_segm(unsigned segm);
int	T6X_set_smp_off();
int	T6X_smp_ptr(unsigned long addr,unsigned area,unsigned *page,void **p);
int	T6X_set_smp_ptr(unsigned long addr,unsigned area,void **p);

//------------------ macros/function for DEV_ID access --------------------

int	T6X_dev_id_data(unsigned *dev_id);
int 	T6X_dev_name(char *);

#define	T62_BOARD	(T6X_id()==T62_DEV_ID)
#define	T67_BOARD	(T6X_id()==T67_DEV_ID)
#define	T62MX_BOARD	(T6X_id()==T62MX_DEV_ID)
#define	T67MX_BOARD	(T6X_id()==T67MX_DEV_ID)

#define T6X_VALID_BOARD	(T6X_id()==T62_DEV_ID || T6X_id()==T67_DEV_ID || T6X_id()==T62MX_DEV_ID || T6X_id()==T67MX_DEV_ID)
#define T6X_BOARD	(T6X_id()==T62_DEV_ID || T6X_id()==T67_DEV_ID)
#define T6XMX_BOARD	(T6X_id()==T62MX_DEV_ID || T6X_id()==T67MX_DEV_ID)


//--------------------- macros for HPI access ----------------------------
unsigned long	T6X_hpic_data();
unsigned long	T6X_hpia_data();
unsigned long	T6X_hpid_data();
unsigned long	T6X_hpid_ainc_data();

#define	T6X_HPIC_BOB_BITMASK		0x1
#define	T6X_HPIC_DSPINT_BITMASK		0x2
#define	T6X_HPIC_HINT_BITMASK		0x4
#define	T6X_HPIC_HRDY_BITMASK		0x8
#define	T6X_HPIC_FETCH_BITMASK		0x10

#define	T6X_HPIA_MIN			0x0L
#define	T6X_HPIA_MAX			0xffffffffL

#define T6X_HPI_INIT			{outpw(T6X_HPIC_RG_LSW,0x1);outpw(T6X_HPIC_RG_MSW,0x1);outpw(T6X_HPIA_RG_LSW,0);outpw(T6X_HPIA_RG_MSW,0);}

#define T6X_HPIC_DATA_LSW		(inpw(T6X_HPIC_RG_LSW) & 0x1f)
#define T6X_HPIC_DATA_MSW		(inpw(T6X_HPIC_RG_MSW) & 0x1f)
#define T6X_HPIC_DATA			T6X_hpic_data()
#define T6X_SET_HPIC(d)			{outpw(T6X_HPIC_RG_LSW,((unsigned)d)&0x17); outpw(T6X_HPIC_RG_MSW,((unsigned)d)&0x17);}

#define T6X_HPIC_DSPINT_DATA		((inpw(T6X_HPIC_RG_LSW))>>1 & 0x1)
#define T6X_HPIC_HINT_DATA		((inpw(T6X_HPIC_RG_LSW))>>2 & 0x1)
#define T6X_HPIC_HRDY_DATA		((inpw(T6X_HPIC_RG_LSW))>>3 & 0x1)
#define T6X_HPIC_FETCH_DATA		((inpw(T6X_HPIC_RG_LSW))>>4 & 0x1)
#define T6X_HPIC_HWOB_DATA		((inpw(T6X_HPIC_RG_LSW))    & 0x1)
#define T6X_SET_HPIC_HWOB		outpw(T6X_HPIC_RG_LSW,0x1)
#define T6X_SET_HPIC_DSPINT		outpw(T6X_HPIC_RG_LSW,inpw(T6X_HPIC_RG_LSW) | 0x2)
#define T6X_CLR_HPIC_HINT		outpw(T6X_HPIC_RG_LSW,inpw(T6X_HPIC_RG_LSW) | 0x4)
#define T6X_SET_HPIC_FETCH		outpw(T6X_HPIC_RG_LSW,inpw(T6X_HPIC_RG_LSW) | 0x10)

#define T6X_HPIA_DATA_LSW		inpw(T6X_HPIA_RG_LSW)
#define T6X_HPIA_DATA_MSW		inpw(T6X_HPIA_RG_MSW)
#define T6X_HPIA_DATA			T6X_hpia_data()

#define T6X_SET_HPIA_LSW(d)		outpw(T6X_HPIA_RG_LSW,(unsigned)((unsigned long)(d & 0xffffL)))
#define T6X_SET_HPIA_MSW(d)		outpw(T6X_HPIA_RG_MSW,(unsigned)(((unsigned long)(d>>16) & 0xffffL)))
#define T6X_SET_HPIA(d)			{T6X_SET_HPIA_LSW(d);T6X_SET_HPIA_MSW(d);}
#define T6X_HPIA_RESET			{outpw(T6X_HPIA_RG_LSW,0);outpw(T6X_HPIA_RG_MSW,0);}
#define T6X_HPIA_CLEAR			T6X_HPI_RESET

#define T6X_HPID_DATA_LSW		inpw(T6X_HPID_RG_LSW)
#define T6X_HPID_DATA_MSW		inpw(T6X_HPID_RG_MSW)
#define T6X_HPID_DATA			T6X_hpid_data()

#define T6X_SET_HPID_LSW(d)		outpw(T6X_HPID_RG_LSW,(unsigned)((unsigned long)d&0xffffL))
#define T6X_SET_HPID_MSW(d)		outpw(T6X_HPID_RG_MSW,(unsigned)((unsigned long)(d>>16)&0xffffL))
#define T6X_SET_HPID(d)			{T6X_SET_HPID_LSW(d);T6X_SET_HPID_MSW(d);}

#define T6X_HPID_AINC_DATA_LSW		inpw(T6X_HPID_AINC_RG_LSW)
#define T6X_HPID_AINC_DATA_MSW		inpw(T6X_HPID_AINC_RG_MSW)
#define T6X_HPID_AINC_DATA		T6X_hpid_ainc_data()

#define T6X_SET_HPID_AINC_LSW(d)	outpw(T6X_HPID_AINC_RG_LSW,(unsigned)((unsigned long)d&0xffffL))
#define T6X_SET_HPID_AINC_MSW(d)	outpw(T6X_HPID_AINC_RG_MSW,(unsigned)((unsigned long)(d>>16)&0xffffL))
#define T6X_SET_HPID_AINC(d)		{T6X_SET_HPID_AINC_LSW(d);T6X_SET_HPID_AINC_MSW(d);}


//------------------- UECM/ECC ----------------------
#define	T6X_EMU_OFF			0
#define	T6X_EMU_IO_BADDR_DEFAULT	0x240
#define	T6X_ECC_IO_BADDR_CNF_QTY	8
#define	T6X_UECM_IO_BADDR_CNF_QTY	4
#define T6X_EMU_MDSP			0
#define T6X_EMU_XDSP			1


int	T6X_emu_io_baddr_data();
int 	T6X_set_emu_io_baddr(unsigned io_addr);
int 	T6X_set_emu_mdsp();
int 	T6X_set_emu_xdsp();
int 	T6X_set_emu_reset();
int 	T6X_emu_path_data();

#define T6X_set_emu_off() 		(T6X_set_emu_xdsp())
#define T6X_set_ecc_off() 		(T6X_set_emu_xdsp())
#define T6X_ECC_OFF	 		(T6X_set_emu_xdsp())
#define T6X_EMU(baddr)			(T6X_set_emu_io_baddr(baddr))
#define T6X_EMU_DEFAULT			(T6X_set_emu_io_baddr(T6X_EMU_IO_BADDR_DEFAULT))
#define T6X_EMU_SELECT_MDSP		(T6X_set_emu_mdsp())
#define T6X_EMU_SELECT_XDSP		(T6X_set_emu_xdsp())
#define T6X_EMU_RESET			(T6X_set_emu_reset())


//------------------- utility functions -----------------------------
int T6X_open(unsigned);
int T6X_close();
int T6X_last_err();
int T6X_err_flag();
int T6X_clr_err_flag();
int T6X_last_err_message(char *);

